%{

(* This program is free software.  See LICENSE for more information *)

let parse_error msg =
print_string msg;;

open Smt_sorts;;
open Smt_tables;;
open Smt_helper_theory;;


%}
%token <string> NUMERAL_TOK
%token <string> VAR_TOK
%token <string> SYM_TOK
%token <string> STRING_TOK
%token <string> AR_SYMB
%token <string> USER_VAL_TOK
%token TRUE_TOK
%token BOGUS
%token FALSE_TOK
%token ITE_TOK
%token NOT_TOK
%token IMPLIES_TOK
%token IF_THEN_ELSE_TOK
%token AND_TOK
%token OR_TOK
%token XOR_TOK
%token IFF_TOK
%token EXISTS_TOK
%token FORALL_TOK
%token LET_TOK
%token FLET_TOK
%token NOTES_TOK
%token SORTS_TOK
%token FUNS_TOK
%token PREDS_TOK
%token EXTENSIONS_TOK
%token DEFINITION_TOK
%token AXIOMS_TOK
%token LOGIC_TOK
%token COLON_TOK
%token LPAREN_TOK
%token RPAREN_TOK
%token SAT_TOK
%token UNSAT_TOK
%token UNKNOWN_TOK
%token THEORY_TOK
%token ASSUMPTION_TOK
%token FORMULA_TOK
%token STATUS_TOK
%token BENCHMARK_TOK
%token EXTRASORTS_TOK
%token EXTRAFUNS_TOK
%token EXTRAPREDS_TOK
%token LANGUAGE_TOK
%token DOLLAR_TOK
%token EQUALS_TOK
%token DISTINCT_TOK
%token SEMICOLON_TOK
%token EOF_TOK

%start theory
%type <Smt_helper_theory.theory> theory

%%
theory:
    LPAREN_TOK THEORY_TOK theory_name theory_attributes RPAREN_TOK
    {
      
	  (
      	let b = THEORY($3,$4) in
      	  if not (Hashtbl.mem symbol_table "sorts") then 
            raise (Validation_error "Sort not defined.")
          else 
      	    if not (Hashtbl.mem symbol_table "definition") then 
              raise (Validation_error "Definition not defined.")
            else 
            (
 	    	  b;
            )
      )
    }

theory_name:
    SYM_TOK 
    { 
    	Printf.printf "Theory_name:%s\n" $1;
		NAME($1)
    }

theory_attributes:
    theory_attribute theory_attributes {$1::$2}
  | theory_attribute {[$1]}

theory_attribute:
    COLON_TOK SORTS_TOK LPAREN_TOK sort_symb_decls RPAREN_TOK 
    {
      if Hashtbl.mem symbol_table "sorts" then 
        raise (Validation_error "More than one sorts definition found.")
      else (
      	Hashtbl.replace symbol_table "sorts" "exists";
      	SORTS($4) 
      )
  
    }
  | COLON_TOK FUNS_TOK LPAREN_TOK fun_symb_decls RPAREN_TOK {FUNS($4)}
  | COLON_TOK PREDS_TOK LPAREN_TOK pred_symb_decls RPAREN_TOK {PREDS($4)}
  | COLON_TOK DEFINITION_TOK STRING_TOK 
    {
      if Hashtbl.mem symbol_table "definition" then 
        raise (Validation_error "More than one definition definition found.")
      else (
      	Hashtbl.replace symbol_table "definition" "exists";
        DEFINITION($3)
        )
    }
   
  | COLON_TOK AXIOMS_TOK LPAREN_TOK an_formulas RPAREN_TOK {AXIOMS($4)}
  | COLON_TOK NOTES_TOK STRING_TOK {NOTES($3)}
  | annotation {ANNOTATION($1)}


sort_symbs:
    sort_symb sort_symbs
    { 
      (* returns a list of sort names *)
      $1::$2
    }
  | sort_symb 
    { 
      (* returns a singleton list of one sort name *)
      [$1]
    }

sort_symb_decls:
    sort_symb_decls sort_symb_decl
    { 
      (* returns a list of sort names *)
      $2::$1
    }
  | sort_symb_decl 
    { 
      (* returns a singleton list of one sort name *)
      [$1]
    }


fun_symb_decls:
    fun_symb_decls fun_symb_decl { $2::$1 }
  | fun_symb_decl { [$1] }







fun_symb_decl:
    LPAREN_TOK fun_symb  sort_symb sort_symbs annotations RPAREN_TOK
    {
      (* returns the signature of a function with annotations *)
        if Hashtbl.mem funs $2 then
          raise (Validation_error ("function "^$2^" already defined."))
        else (
       	  Hashtbl.add funs $2 $2;
          let args = List.rev(List.tl(List.rev($3::$4))) in
          let ret = List.hd(List.rev($3::$4)) in
          Hashtbl.add all_sorts $2 ret;
       	  let env = Hashtbl.find environment "env" in 
       	  let fun_sort = Smt_sorts.FUNCTION_SORT($2,args,ret) in
       	  Hashtbl.replace environment "env" (fun_sort::env);
          FUNC_SIG($2,args,ret,$5))
    }
  | LPAREN_TOK fun_symb  sort_symb sort_symbs RPAREN_TOK
    {
      (* returns the signature of a function without annotations *)
        if Hashtbl.mem funs $2 then
          raise (Validation_error ("function "^$2^" already defined."))
        else (
          Hashtbl.add funs $2 $2;
          let args = List.rev(List.tl(List.rev($3::$4))) in
          let ret = List.hd(List.rev($3::$4)) in
          Hashtbl.add all_sorts $2 ret;
       	  let env = Hashtbl.find environment "env" in 
       	  let fun_sort = Smt_sorts.FUNCTION_SORT($2,args,ret) in
       	  Hashtbl.add environment "env" (fun_sort::env);
          FUNC_SIG($2,args,ret,[]))
    }
  | LPAREN_TOK fun_symb sort_symb annotations RPAREN_TOK
    {
      (* returns the signature of a function with annotations *)
        if Hashtbl.mem funs $2 then
          raise (Validation_error ("function "^$2^" already defined."))
        else (
          Hashtbl.add funs $2 $2;
          Hashtbl.add all_sorts $2 $3;
       	  let env = Hashtbl.find environment "env" in 
       	  let fun_sort = Smt_sorts.FUNCTION_SORT($2,[],$3) in
       	  Hashtbl.add environment "env" (fun_sort::env);
          FUNC_SIG($2,[],$3,$4))
    }
  | LPAREN_TOK fun_symb sort_symb RPAREN_TOK
    {
      (* returns the signature of a function without annotations *)
        if Hashtbl.mem funs $2 then
          raise (Validation_error ("function "^$2^" already defined."))
        else (
          Hashtbl.add funs $2 $2;
          Hashtbl.add all_sorts $2 $3;
       	  let env = Hashtbl.find environment "env" in 
       	  let fun_sort = Smt_sorts.FUNCTION_SORT($2,[],$3) in
       	  Hashtbl.add environment "env" (fun_sort::env);
          FUNC_SIG($2,[],$3,[]))
    }
  	
pred_symb_decls:
    pred_symb_decls pred_symb_decl {$2::$1}
  | pred_symb_decl {[$1]}

pred_symb_decl:
    LPAREN_TOK pred_symb sort_symbs annotations RPAREN_TOK
    {
      (* returns the signature of a predicate with annotations *)
        if Hashtbl.mem preds $2 then
          raise (Validation_error ("predicate "^$2^" already defined."))
        else (
	  	  Hashtbl.add preds $2 $2;
	  	  Hashtbl.add all_sorts $2 "_boolean_";
      	  let env = Hashtbl.find environment "env" in 
       	  let pred_sort = Smt_sorts.PREDICATE_SORT($2,$3) in
       	  Hashtbl.replace environment "env" (pred_sort::env);
	  PRED_SIG($2,$3,$4))
    }
  | LPAREN_TOK pred_symb annotations RPAREN_TOK
    {
      (* returns the signature of a predicate with annotations *)
      if Hashtbl.mem preds $2 then
        raise (Validation_error ("predicate "^$2^" already defined."))
      else (
      	Hashtbl.add preds $2 $2;
        Hashtbl.add all_sorts $2 "_boolean_";      	
      	  let env = Hashtbl.find environment "env" in 
       	  let pred_sort = Smt_sorts.PREDICATE_SORT($2,[]) in
       	  Hashtbl.replace environment "env" (pred_sort::env);
        PRED_SIG($2,[],$3))
    }
  | LPAREN_TOK pred_symb sort_symbs RPAREN_TOK
    {
      (* returns the signature of a predicate without annotations *)
      if Hashtbl.mem preds $2 then
        raise (Validation_error ("predicate "^$2^" already defined."))
      else (
      	Hashtbl.add preds $2 $2;
        Hashtbl.add all_sorts $2 "_boolean_";      	
      	  let env = Hashtbl.find environment "env" in 
       	  let pred_sort = Smt_sorts.PREDICATE_SORT($2,$3) in
       	  Hashtbl.replace environment "env" (pred_sort::env);
        PRED_SIG($2,$3,[]))
    }
  | LPAREN_TOK pred_symb  RPAREN_TOK
    {
      (* returns the signature of a predicate without annotations *)
      if Hashtbl.mem preds $2 then
        raise (Validation_error ("predicate "^$2^" already defined."))
      else (
      	Hashtbl.add preds $2 $2;
        Hashtbl.add all_sorts $2 "_boolean_";      	
      	  let env = Hashtbl.find environment "env" in 
       	  let pred_sort = Smt_sorts.PREDICATE_SORT($2,[]) in
       	  Hashtbl.replace environment "env" (pred_sort::env);
        PRED_SIG($2,[],[]))
    }



an_formulas:
    an_formula an_formulas {$1::$2}
  | an_formula {[$1]}

an_formula:
    an_atom {$1}
  | LPAREN_TOK connective an_formulas annotations RPAREN_TOK
    {
      (* returns a formula for one of the boolean connectives with annotations *)
      match ($2,$3) with
        (NOT_TOK,[expr1]) -> 
            FA(NOT(expr1),$4)
      | (IMPLIES_TOK, [expr1;expr2]) -> 
            FA(IMPLIES(expr1,expr2),$4)
      | (IF_THEN_ELSE_TOK, [expr1; expr2; expr3]) -> 
            FA(IF_THEN_ELSE(expr1,expr2,expr3),$4)
      | (AND_TOK, exprl) -> 
            FA(AND(exprl),$4)
      | (OR_TOK, exprl) -> 
            FA(OR(exprl),$4)
      | (XOR_TOK, exprl)-> 
            FA(XOR(exprl),$4)
      | (IFF_TOK, exprl)-> 
            FA(IFF(exprl),$4)
      | _ -> raise Connective_rank_mismatch (* means that a connective was used with a number of arguments that doesn't make sense *)
    }
  | LPAREN_TOK connective an_formulas RPAREN_TOK
    {
      (* returns a formula for one of the boolean connectives without annotations *)
      match ($2,$3) with
        (NOT_TOK,[expr1]) -> 
            F(NOT(expr1))
      | (IMPLIES_TOK, [expr1;expr2]) -> 
            F(IMPLIES(expr1,expr2))
      | (IF_THEN_ELSE_TOK, [expr1; expr2; expr3]) -> 
            F(IF_THEN_ELSE(expr1,expr2,expr3))
      | (AND_TOK, exprl) -> 
            F(AND(exprl))
      | (OR_TOK, exprl) -> 
            F(OR(exprl))
      | (XOR_TOK, exprl)-> 
            F(XOR(exprl))
      | (IFF_TOK, exprl)-> 
            F(IFF(exprl))
      | _ -> raise Connective_rank_mismatch (* means that a connective was used with a number of arguments that doesn't make sense *)
      
    }
  | LPAREN_TOK quant_symb quant_vars an_formula annotations RPAREN_TOK
    {
      (* returns a quantified formula with annotations *)
      FA(QUANT($2,$3,$4),$5)
    }
  | LPAREN_TOK quant_symb quant_vars an_formula RPAREN_TOK
    {
      (* returns a quantified formula with no annotations *)
      F(QUANT($2,$3,$4))
    }
  | LPAREN_TOK LET_TOK LPAREN_TOK VAR_TOK an_term RPAREN_TOK an_formula annotations RPAREN_TOK
    {
      (* returns a let formula with annotations *)
      FA(LET($4,$5,$7),$8)
    }
  | LPAREN_TOK LET_TOK LPAREN_TOK VAR_TOK an_term RPAREN_TOK an_formula RPAREN_TOK
    {
      (* returns a let formula with no annotations *)
      F(LET($4,$5,$7))
    }
  | LPAREN_TOK FLET_TOK LPAREN_TOK fvar an_formula RPAREN_TOK an_formula annotations RPAREN_TOK
    {
      (* returns a flet formula with annotations *)
      FA(FLET($4,$5,$7),$8)
    }
  | LPAREN_TOK FLET_TOK LPAREN_TOK fvar an_formula RPAREN_TOK an_formula RPAREN_TOK
    {
      (* returns a flet formula with no annotations *)
      F(FLET($4,$5,$7))
    }

quant_vars:
    quant_vars quant_var 
    {
      (* returns a list of binders *)
      $2::$1
    }
  | quant_var 
    {
      (* returns a singleton list with one binder in it *)
      [$1]
    }

quant_var: 
    LPAREN_TOK VAR_TOK sort_symb RPAREN_TOK
    {
      (* returns the name of the variable along with the name of its sort *)
      ($2,$3)
    }

quant_symb:
    EXISTS_TOK {EXISTS}
  | FORALL_TOK {FORALL}

connective:
    NOT_TOK {NOT_TOK}
  | IMPLIES_TOK {IMPLIES_TOK}
  | IF_THEN_ELSE_TOK {IF_THEN_ELSE_TOK}
  | AND_TOK {AND_TOK}
  | OR_TOK {OR_TOK}
  | XOR_TOK {XOR_TOK}
  | IFF_TOK {IFF_TOK}


an_atom:
    prop_atom 
    {
      (* returns a formula for a propositional atom with no annotations *)
      F($1)
    }
  | LPAREN_TOK prop_atom annotations RPAREN_TOK 
    {
      (* returns a formula for a propositional atom with annotations *)
      FA($2,$3)
    }
  | LPAREN_TOK pred_symb an_terms annotations RPAREN_TOK
    {
      (* returns a formula for a predicate that has 1 or more arguments with annotations*)
      (
        if not (Hashtbl.mem preds $2) then
          raise (Validation_error ("predicate "^$2^" not yet defined."));
        FA(PRED($2,$3),$4)
      )
    }
  | LPAREN_TOK pred_symb an_terms RPAREN_TOK
    {
      (* returns a formula for a predicate that has 1 or more arguments with no annotations*)
      (
        if not (Hashtbl.mem preds $2) then
          raise (Validation_error ("predicate "^$2^" not yet defined."));
        F(PRED($2,$3))
      )
    }
  | LPAREN_TOK EQUALS_TOK an_term an_terms annotations RPAREN_TOK
    {
      (* returns a formula for equality with annotations
         Note that it requires at least two terms*)
      FA(EQUALS($3::$4),$5)
    }
  | LPAREN_TOK EQUALS_TOK an_term an_terms RPAREN_TOK
    {
      (* returns a formula for equality with no annotations
         Note that it requires at least two terms*)      
      F(EQUALS($3::$4))
    }
  | LPAREN_TOK DISTINCT_TOK an_term an_terms annotations RPAREN_TOK
    {
      (* returns a formula for distinct with annotations
         Note that it requires at least two terms*)
      FA(DISTINCT($3::$4),$5)
    }
  | LPAREN_TOK DISTINCT_TOK an_term an_terms RPAREN_TOK
    {
      (* returns a formula for distinct with no annotations
         Note that it requires at least two terms*)
      F(DISTINCT($3::$4))
    }


prop_atom:
  | TRUE_TOK
    {
      (* returns a formula for true *)
      TRUE_PRED
    }
  | FALSE_TOK
    { 
      (* returns a formula for false *)
      FALSE_PRED

    }
  | fvar
    {
      (* returns a formula variable that should be bound with a flet formula higher in the parse tree *)
      FVAR($1)
    }
  | pred_symb 
    {
      (* returns a predicate with no arguments (a propositional variable) *)
      (
        if not (Hashtbl.mem preds $1) then
          raise (Validation_error ("predicate "^$1^" not yet defined."));
        PRED($1,[])
      )
    }
  

an_terms:
    an_term an_terms 
    {
      (* returns a list of terms*)
      $1::$2
    }
  | an_term 
    { 
      (* returns a singleton list of terms *)
      [$1]
    }

an_term:
    basic_term 
    { 
      (* returns a basic term with no annotations *)
      T($1)
    }
  | LPAREN_TOK basic_term annotations RPAREN_TOK 
    {
      (* adds annotations to a basic term and returns an annotated term *)
      TA($2,$3)
    }
  | LPAREN_TOK fun_symb an_terms annotations RPAREN_TOK
    {
      (* returns a function term with at least one argument with annotations *)
      (
        if not (Hashtbl.mem funs $2) then
          raise (Validation_error ("function "^$2^" not yet defined."));
        TA(FUN($2,$3),$4)
      )
    }
  | LPAREN_TOK fun_symb an_terms RPAREN_TOK
    {
      (* returns a function term with at least one argument with no annotations *)
      (
        if not (Hashtbl.mem funs $2) then
          raise (Validation_error ("function "^$2^" not yet defined."));
        T(FUN($2,$3))
      )
    }
  | LPAREN_TOK ITE_TOK an_formula an_term an_term annotations RPAREN_TOK
    {
      (* returns an ITE term with annotations *)
      TA(ITE($3,$4,$5),$6)
    }
  | LPAREN_TOK ITE_TOK an_formula an_term an_term RPAREN_TOK
    {
      (* returns an ITE term with no annotations *)
      T(ITE($3,$4,$5))
    }


basic_term:
    VAR_TOK 
    { 
      (* returns a variable term*)
      VAR($1)
            
    }
  | NUMERAL_TOK 
    {
      (* returns a numeral term*)
      NUMERAL(Big_int.big_int_of_string $1)
    }
  | fun_symb 
    {
      (* returns a function term with no arguments (a constant)*)
      (
        if not (Hashtbl.mem funs $1) then
          raise (Validation_error ("function "^$1^" not yet defined."));
        FUN($1,[])
      )
    }

annotations:
    annotation annotations 
    { 
      (*  returns a list of annotations*)
      ($1::$2)
    }
  | annotation 
    {
      (* returns a singleton list of annotations *)
      [$1]
    }
  



annotations:
    annotation annotations 
    { 
      (*  returns a list of annotations*)
      ($1::$2)
    }
  | annotation 
    {
      (* returns a singleton list of annotations *)
      [$1]
    }
  
annotation:
    attribute 
    {
      (* returns the attribute's name and None because it doesn't have a corresponding value *)
      ($1,None)
    }
  | attribute user_value 
    {
      (* returns the attribute's name along with its corresponding value*)
      ($1,Some $2)
    }

user_value:
    USER_VAL_TOK
    {
    	$1
    }

sort_symb_decl:
    SYM_TOK 
    { 
      (* returns the name of the sort symbol*)
      (
        Hashtbl.add sorts $1 $1;
        $1 
      )
    }

sort_symb:
    SYM_TOK 
    { 
      (* returns the name of the sort symbol*)
      
      (
        if not (Hashtbl.mem sorts $1) then
	      raise (Validation_error ("sort "^$1^" is not defined."));
        $1
      )
    }

pred_symb:
    SYM_TOK 
    {
        (* returns the name of the predicate *)
        $1
    }
  | AR_SYMB 
    {
        (* returns the name of the predicate  *)
        $1
    }

fun_symb:
    SYM_TOK
    {
        (* returns the name of the function *)
        $1
    }
  | AR_SYMB 
    { 
        (* returns the name of the function *)
        $1
    }
  | NUMERAL_TOK
    {
        (* returns the name of the function *)
        $1
    }


attribute:
    COLON_TOK SYM_TOK { ":"^ $2}

fvar:
    DOLLAR_TOK SYM_TOK { "$" ^ $2}
    
